@@ -1,4 +1,5 @@
 from past.builtins import basestring
+import argparse
 import os.path
 import re
 import errno
@@ -7,15 +8,16 @@ import stat
 import getpass
 from builtins import input
 from operator import attrgetter
-import netrc
+from netrc import netrc
 import xnat
 from xnat.exceptions import XNATResponseError
 from .exceptions import (
     XnatUtilsLookupError, XnatUtilsUsageError, XnatUtilsKeyError,
     XnatUtilsNoMatchingSessionsException,
-    XnatUtilsSkippedAllSessionsException)
+    XnatUtilsSkippedAllSessionsException, XnatUtilsError)
 import warnings
 import logging
+from .version_ import __version__
 
 logger = logging.getLogger('xnat-utils')
 
@@ -51,11 +53,11 @@ illegal_scan_chars_re = re.compile(r'\.')
 
 session_modality_re = re.compile(r'\w+_\w+_([A-Z]+)\d+')
 
-server_name_re = re.compile(r'(http://|https://)?([\w\-\.]+).*')
+server_name_re = re.compile(r'(https?://)?([\w\-\.]+).*')
 
 
-def connect(user=None, loglevel='ERROR', connection=None, server=None,
-            use_netrc=True, failures=0):
+def connect(server=None, user=None, loglevel='ERROR', connection=None,
+            use_netrc=True, failures=0, password=None):
     """
     Opens a connection to MBI-XNAT
 
@@ -82,6 +84,9 @@ def connect(user=None, loglevel='ERROR', connection=None, server=None,
     use_netrc : bool
         Whether to load and save user credentials from netrc file
         located at $HOME/.netrc
+    password : str
+        Password provided to login. Will be ignored unless 'user' and 'server'
+        are not also provided
     Returns
     -------
     connection : xnat.Session
@@ -89,43 +94,59 @@ def connect(user=None, loglevel='ERROR', connection=None, server=None,
     """
     if connection is not None:
         return WrappedXnatSession(connection)
-    netrc_path = os.path.join(os.path.expanduser('~'), '.netrc')
-    save_netrc = False
-    password = None
-    if use_netrc:
-        try:
-            saved_servers = netrc.netrc(netrc_path).hosts
-        except Exception:
-            saved_servers = {}
+    netrc_path = os.path.join(os.path.expanduser('~'),
+                              ('.netrc' if os.name != 'nt' else '_netrc'))
+    netrc_match = False
+    # Extract server name from netrc file if 'use_netrc' flag is set and either
+    # the server is not provided or it doesn't include the protocol (in which
+    # case it will be considered as a potential name fragment)
+    if use_netrc and os.path.exists(netrc_path):
+        # Read netrc file and return the server addresses saved within it
+        with open(netrc_path) as f:
+            lines = f.read().split('\n')
+        server_names = []
+        for line in lines:
+            if line.startswith('machine'):
+                server_names.append(line.split()[-1])
+        if not server_names:
+            raise XnatUtilsError(
+                "Malformed Netrc file ({}), please delete or flag "
+                "use_netrc==False")
+        if server is None:
+            # Default to the first saved server
+            server = server_names[0]
+            netrc_match = True
+        else:
+            # Check whether the provided server name is in the netrc or not
+            protocol, dn = server_name_re.match(server).groups()
+            # If protocol (e.g. http://) is included in server name, treat as
+            # a complete hostname
+            if protocol is not None:
+                netrc_match = dn in server_names
+            # Otherwise treat host name as a potential fragment that can
+            # match any of the save servers
+            else:
+                matches = [s for s in server_names if dn in s]
+                if len(matches) == 1:
+                    server = matches[0]
+                    netrc_match = True
+                elif len(matches) > 1:
+                    raise XnatUtilsUsageError(
+                        "Given server name (or part thereof) '{}' matches "
+                        "multiple servers in ~/.netrc file ('{}')".format(
+                            server, "', '".join(matches)))
+        saved_servers = netrc().hosts
     else:
         saved_servers = {}
-    if server is None and saved_servers:
-        # Get default server from first line in netrc file
-        with open(netrc_path) as f:
-            server = f.readline().split()[-1]
-    else:
-        matches = [s for s in saved_servers if server in s]
-        if len(matches) == 1:
-            server = matches[0]
-        elif len(matches) > 1:
-            raise XnatUtilsUsageError(
-                "Given server name (or part thereof) '{}' matches "
-                "multiple servers in ~/.netrc file ('{}')".format(
-                    server, "', '".join(matches)))
-        else:
-            if server is None:
-                server = input('XNAT server URL: ')
-            # A little hack to avoid issues with the redirection
-            # from monash.edu to monash.edu.au
-            if server.endswith('monash.edu'):
-                server += '.au'
-            if user is None:
-                user = input("XNAT username for '{}': ".format(server))
+    if server is None:
+        server = input(
+            'XNAT server hostname (e.g. mbi-xnat.erc.monash.edu.au): ')
+    if not netrc_match:
+        if user is None:
+            user = input("XNAT username for '{}': ".format(server))
+        if password is None:
             password = getpass.getpass()
-            if use_netrc:
-                save_netrc = True
-    # Ensure that the protocol is added to the server URL
-    # FIXME: Should be able to handle http:// protocols as well.
+    # Prepend default HTTP protcol if protocol is not present
     if server_name_re.match(server).group(1) is None:
         server = 'http://' + server
     kwargs = {'user': user, 'password': password}
@@ -148,11 +169,15 @@ def connect(user=None, loglevel='ERROR', connection=None, server=None,
             logger.warning(msg)
             try:
                 del saved_servers[server_name_re.match(server).group(2)]
-                write_netrc(netrc_path, saved_servers)
-                logger.warning("Removed saved credentials for {}..."
-                               .format(server))
             except KeyError:
                 pass
+            else:
+                if saved_servers:
+                    write_netrc(netrc_path, saved_servers)
+                else:
+                    os.remove(netrc_path)
+                logger.warning("Removed saved credentials for {}..."
+                               .format(server))
             if failures < 3:
                 return connect(server=server, loglevel=loglevel,
                                connection=connection,
@@ -164,7 +189,7 @@ def connect(user=None, loglevel='ERROR', connection=None, server=None,
                     "blocked for 1 hour. Please contact "
                     "your administrator to reset.".format(user))
         else:
-            if save_netrc:
+            if use_netrc and not netrc_match:
                 alias, secret = connection.services.issue_token()
                 # Strip protocol (i.e. https://) from server
                 server_name = server_name_re.match(server).group(2)
@@ -395,16 +420,13 @@ def matching_sessions(login, session_ids, with_scans=None,
                     raise XnatUtilsKeyError(
                         id_, "No subject named '{}'".format(id_))
                 sessions.update(subject.experiments.values())
-            elif id_ .count('_') >= 2:
+            else:
                 try:
                     session = base.experiments[id_]
                 except KeyError:
                     raise XnatUtilsKeyError(
                         id_, "No session named '{}'".format(id_))
                 sessions.add(session)
-            else:
-                raise XnatUtilsKeyError(
-                    id_, "Invalid ID '{}' for listing sessions".format(id_))
     filtered = [s for s in sessions if valid(s)]
     if not filtered:
         raise XnatUtilsNoMatchingSessionsException(
@@ -509,5 +531,42 @@ def print_response_error(e):
 
 class DummyContext(object):
 
-    def __exit__(self):
+    def __exit__(self, *args):
         pass
+
+
+def base_parser(description):
+    parser = argparse.ArgumentParser(
+        description=description,
+        formatter_class=argparse.RawTextHelpFormatter)
+    return parser
+
+
+def add_default_args(parser):
+    parser.add_argument('--user', '-u', type=str, default=None,
+                        help=("The user to connect to MBI-XNAT with"))
+    parser.add_argument('--version', '-V', action='version',
+                        version='%(prog)s ' + __version__)
+    parser.add_argument('--server', '-s', type=str, default=None,
+                        help=("The XNAT server to connect to. If not "
+                              "provided the first server found in the "
+                              "~/.netrc file will be used, and if it is "
+                              "empty the user will be prompted to enter an "
+                              "address for the server. Multiple URLs "
+                              "stored in the ~/.netrc file can be "
+                              "distinguished by passing part of the URL"))
+    parser.add_argument('--loglevel', type=int, default=logging.INFO,
+                        help="The logging level to use")
+    parser.add_argument('--no_netrc', '-n', action='store_true',
+                        default=False,
+                        help=("Don't use or store user access tokens in "
+                              "~/.netrc. Useful if using a public account"))
+
+
+def set_logger(level=logging.INFO):
+    handler = logging.StreamHandler()
+    handler.setLevel(level)
+    formatter = logging.Formatter("%(levelname)s - %(message)s")
+    handler.setFormatter(formatter)
+    logger.addHandler(handler)
+    return logger
